/****************************************************************************
*                             M8P0327.C
*      demonstrates continuous measurement and transfer to the PC
*
*              version                       : 1.D.
*              creation                      : 27.03.1996
*              last change                   : 07.06.2000
*              author                        : H. Klinkenborg
*                                              A. Horst
*
*    (c) by SORCUS Computer GmbH Heidelberg 1996
****************************************************************************/

/****************************************************************************
*  Description: This program measures continuously analog input from up to
*        16 channels of the module M-AD12-16 and puts the data into a buffer
*        of the operating system OsX of the MODULAR-4/486 board. If the
*        buffer has reached a limit, the task sends a SRQ to the PC.
*
*  Compiler: Borland C 3.1
*
*  hardware requirements: PC with monitor, MODULAR-4/486 board, SPB-module
*        M-AD12-16
*
*  software requirements: This program requires the library ML8RTBIB.OBJ
*        and the header ML8RTBIB.H. The operating system OSX on the
*        MODULAR-4/486 board must be ML8-3A.00x or greater
****************************************************************************/

/****************************************************************************
*  history:
*        1.A. 27.03.1996 10:30; HK
*        - creation of the file
*        1.B. 19.12.1996 15:30; AH
*        - only one task, sends buffered SRQ
*        1.C. 18.11.1998 11:57; HK (use ML8RTBIB.LIB)
*        1.D. 07.06.2000 11:57; HK (newest PDT-configuration)
****************************************************************************/

/****************************************************************************
*  Includes
****************************************************************************/

#include <dos.h>		       /* Include the default C libraries  */

#include "ml8rtbib.h"                  /* header of real-time-library      */
				       /* requires Turbo-Assembler         */

#include "m8p0327.h"                   /* Include the header file          */


/****************************************************************************
*  Defines
****************************************************************************/

#define PROGRAM_NUMBER  0x327          /* program-number = 327h            */
#define VERSION         '1'            /* program-version                  */
#define REVISION        'D'            /* program-revision                 */

				       /* global values                    */
par_type_M8P0327 parameter;            /* Parameters of program M8P0327    */

ushort mba;

short data_var[16];                    /* Local Parameters: Array used for */
				       /* analog input (16 values maximum) */

void PFAR main_task(void);                  /* prototypes of the procedures     */
void PFAR auto_init(void);
void PFAR start(void);
void PFAR stop(void);
void PFAR read_buf(ushort task, ushort insize,
			 ushort *indata, ushort outsize, ushort *outdata);


/****************************************************************************
*  auto_init
*        initializes the parameters of the program and checks the system
*        parameters, whether the correct timer and interrupt is used and the
*        wanted time constant
*  parameter:     none
****************************************************************************/

void PFAR auto_init(void)                   /* AUTO-INIT-PROCEDURE              */
{
 ushort i;                             /* for-loop variable                */

 ml8rt_entry();                        /* save & prepare registers         */

 parameter.status = INSTALLED;         /* set status = 0                   */
 parameter.rate = 5;                   /* n timer-tics                     */
 parameter.timer_tic = 100;            /* 100 us interval for TIMER-C      */
 parameter.mode = 1;                   /* duration = number of counts      */
 parameter.count = 10000;              /* 10 cycles                        */
 parameter.micro_slot  = 4;            /* here M-AD12-16 is plugged to     */
 parameter.buffer_size = 512;          /* size of system buffer            */
 parameter.no_of_channels = 16;        /* no. of channels = 16             */
 parameter.settletime = 0xFF;          /* settle-time (x 62,5ns)           */
 parameter.srq_limit = 256;            /* no. of bytes until release SRQ   */
 parameter.srq_word = 0x0080;          /* word to be sent as SRQ to PC     */
 parameter.trans_status = 0;           /* transfer status init to 0        */

				       /* no. of bytes to transfer to buf. */
 parameter.trans_size = parameter.no_of_channels * 2;
 for(i = 0; i < 16; i++)
   data_var[i] = 0;	               /* set data array to 0              */

				       /* init. M-AD12-16                  */
  if(parameter.micro_slot < 5)
   mba = parameter.micro_slot * 0x100 + 0x300;
 else
   mba = parameter.micro_slot * 0x020 + 0x320;

 outportb(mba + 0x0b, 0x72);           /* set mode of M-AD12-16/2          */
				       /* set settle-timer                 */
 outportb(mba + 0x07, parameter.settletime);
 outportb(mba + 0x05, 0x00);           /* set gain correction              */
 outportb(mba + 0x06, 0x00);           /* set offset correction            */
				       /* set last channel                 */
 outport (mba + 0x08, parameter.no_of_channels - 1);


 ml8rt_exit();                         /* rebuild registers                */
}

/****************************************************************************
*  start
*        creates the system buffer used for storing the analog input values
*        and activates the TI- and NI-Task
*  parameter:     none
****************************************************************************/

void PFAR start(void)                       /* START-PROCEDURE                  */
{
 ml8rt_entry();                        /* save & prepare registers         */

 parameter.status = RUNNING;           /* set status to 2 (running)        */

				       /* size of buffer on MODULAR-4      */
 parameter.buffer_size = parameter.srq_limit * 2;

				       /* create system buffer             */
 parameter.bufnr = ml8rt_create_buffer(parameter.tdt.task,
				       3,
				       2,
				       0,
				       &parameter.buffer_size);

				       /* no. of bytes to write to buffer  */
 parameter.trans_size   = parameter.no_of_channels * 2;
 parameter.trans_status = 0;           /* status of SRQ data exchange      */

				       /* init. M-AD12-16                  */
 if(parameter.micro_slot < 5)
   mba = parameter.micro_slot * 0x100 + 0x300;
 else
   mba = parameter.micro_slot * 0x020 + 0x320;

 outportb(mba + 0x0B, 0x72);           /* set mode of M-AD12-16/2          */
				       /* set settle-timer                 */
 outportb(mba + 0x07, parameter.settletime);
 outportb(mba + 0x05, 0x00);           /* set gain correction              */
 outportb(mba + 0x06, 0x00);           /* set offset correction            */
				       /* set last channel                 */
 outport (mba + 0x08, parameter.no_of_channels - 1);


				       /* activate TI-Task                 */
				       /* duration = number of counts      */
				       /*            or infinite (mode=0)  */
 ml8rt_wakeup_ti_task(parameter.tdt.task,
		      0,
		      parameter.count * (unsigned long)parameter.mode,
		      (unsigned long)parameter.rate,
		      0L);

 ml8rt_exit();                         /* rebuild registers                */
}

/****************************************************************************
*  stop
*        deactivates the TI- and the NI-Task and sets the status = 4
*  parameter:     none
****************************************************************************/

void PFAR stop(void)                        /* STOP-PROCEDURE                   */
{
 ml8rt_entry();                        /* save & prepare registers         */
				       /* deactivates the TI-Task          */
 ml8rt_sleep_ti_task(parameter.tdt.task);
 parameter.status = CANCELLED;         /* status = 4 (cancelled)           */

 ml8rt_exit();                         /* rebuild registers                */
}

/****************************************************************************
*  read_buf
*        reads the data from the system buffer and transfers them to the PC
*  parameter:
*        task:    no. of task (not used)
*        insize:  no. of transferred data to the function (not used)
*        indata:  transferred data (not used)
*        outsize: expected number of bytes to transfer to the PC
*        outdata: data to transfer to the PC
****************************************************************************/

void PFAR read_buf(ushort task, ushort insize,
			  ushort *indata, ushort outsize, ushort *outdata)
{
 short error;                          /* error variable                   */

 ml8rt_entry_function();               /* save and prepare registers       */
				       /* read data from system buffer     */

 error = ml8rt_read_buffer_block(parameter.bufnr, outsize, outdata);

 parameter.trans_status &= 0xFE;       /* clear acknowledge                */
				       /* return to calling procedure      */
 ml8rt_exit_function(0, outsize, error);
}



/****************************************************************************
*  main_task
*        This main procedure is called by the TI-task scheduler of the
*        operating system OsX. It converts 16 A/D-channels and stores the
*        data into a ring buffer of the OsX.
*  parameter:     none
****************************************************************************/

void PFAR main_task(void)                   /* MAIN-PROCEDURE of TI-Task        */
{
 byte  i;
 ulong usedsize;                       /* number of valid byte in buffer   */
 ulong freesize;                       /* number of free byte in buffer    */

 ml8rt_entry();                        /* save & prepare registers         */

				       /* convert 16 A/D-channels & store  */
				       /* the data in data_var             */
 for(i = 0; i < parameter.no_of_channels; i ++)
   data_var[i] = inport(mba + 0x08);

				       /* move data to the system buffer   */
 parameter.error = ml8rt_write_buffer_block(parameter.bufnr,
					    parameter.trans_size,
					    &data_var);

 if(parameter.error != 0)              /* if error ...                     */
   {
    ml8rt_send_buffer_srq((parameter.srq_word + 1), 1);
   }
 else                                  /* if no error ...                  */
   { 				       /* if the PC has acknowledged the   */
				       /* last service request, check if a */
				       /* new service request can be send  */
    if((parameter.trans_status & 1) == 0)
      {                                /* read buffer status to get size   */
				       /* of valid data in the buffer      */
       ml8rt_get_buffer_status(parameter.bufnr, &freesize, &usedsize);

				       /* If the number of bytes is > or = */
				       /* the defined limit, a service     */
				       /* request will be send             */
       if(usedsize >= parameter.srq_limit)
	 {
	  ml8rt_send_buffer_srq(parameter.srq_word, 1);
				       /* set flag, that service request   */
				       /* has to be acknowledged by the PC */
	  parameter.trans_status |= 1;
	 }
      }
   }
 ml8rt_exit();                         /* rebuild registers                */
}

void main ()                           /* PREPARE                          */
{
 pdt.pdt_head.ucType          = 1;                   /* set PDT-type (only type 1 is     */
				                                         /* supported at the moment)         */
 pdt.pdt_head.ucSize          = sizeof(ML8_PDT_HEAD);/* length of PDT-header             */
				                                         /* (at the moment always = 48)      */
 pdt.pdt_head.usGlobalProc    = 6;                   /* number of procedures             */
 pdt.pdt_head.usProgNo        = PROGRAM_NUMBER;      /* program-number                   */
 pdt.pdt_head.cVersion        = VERSION;             /* program-version                  */
 pdt.pdt_head.cRevision       = REVISION;            /* program-revision                 */
 pdt.pdt_head.ucCPU           = _486DX;              /* CPU-type                         */
 pdt.pdt_head.ucCoProc        = _487DX;              /* Co-processor-type                */
 pdt.pdt_head.ucLanguage      = _CPP31;              /* programming language = C++       */
 pdt.pdt_head.ucProgType      = _USER_PRG;           /* program-type: user-program       */
 pdt.pdt_head.usFlags         = 0;                   /* flags                            */
 pdt.pdt_head.usFlags         = _TI_TASK + _PDT_INFO_ENABLE + _LOCAL_DATA +
		                          _LOCAL_PARAMETER + _FIXED_DATASIZE;
 pdt.pdt_head.usInt           = IRQ_TIMER_C;         /* interrupt-number for this prg.   */
				                                         /* (Timer-C = 93h, TI-TASK)         */

 pdt.pdt_head.ulDataAdr       = 0;                   /* address of data-area             */
				                                         /* (not used here)                  */
 pdt.pdt_head.ulDataSize      = 0;                   /* size of data-area                */
 pdt.pdt_head.ulDataSizeMin   = 0;                   /* minimum size of data-area        */
 pdt.pdt_head.ulDataSizeMax    = 0;                  /* maximum size of data-area        */
				                                         /* calculate                        */
				                                         /* address of parameter-area        */
 ml8rt_phys_adr (&parameter, &pdt.pdt_head.ulGlobalParAdr);
 pdt.pdt_head.ulGlobalParAdr = pdt.pdt_head.ulGlobalParAdr + sizeof(tdt_type);
				                                         /* set number of parameters         */
 pdt.pdt_head.usGlobalParSize = sizeof(parameter) - sizeof(tdt_type);
 pdt.pdt_head.ulHyperAdr      = 0;                   /* addr. of hypertext (not used)    */

 pdt.main_proc                = main_task;           /* addr. of main-procedure          */
 pdt.auto_init                = auto_init;           /* addr. of Auto-Init-proc.         */
 pdt.start                    = start;               /* addr. of Start-procedure         */
 pdt.stop                     = stop;                /* addr. of Stop-procedure          */
 pdt.readbuf                  = read_buf;            /* addr. of special procedure       */
 ml8rt_set_pdt_adr(&pdt);                            /* operating system needs the       */
}				       /* address of the PDT               */




